/* This file was generated by syncqt. */
#ifndef QT_QTQUICK3DGLSLPARSER_VERSION_H
#define QT_QTQUICK3DGLSLPARSER_VERSION_H

#define QTQUICK3DGLSLPARSER_VERSION_STR "6.11.1"

#define QTQUICK3DGLSLPARSER_VERSION 0x0601101

#endif // QT_QTQUICK3DGLSLPARSER_VERSION_H
